@@ -3,30 +3,72 @@
 import gzip
 import json
 import logging
-import os
 import re
 from datetime import datetime
+from pathlib import Path
 from typing import Dict, Optional, Union
 
 import requests
 import xmltodict
-
-
-from aop2db.constants import AOP_ID, AOP_XML_DOWNLOAD, APPLICABILITY, CREATION, CREATION_TIMESTAMP, EVIDENCE, ID, \
-    LAST_MODIFIED, LIFESTAGE, LM_TIMESTAMP, REFERENCES, SEX, TAXONOMY, TAX_ID_LOOKUP
-from aop2db.defaults import AOP_XML_FILE, TAXONOMY_CACHE
-from aop2db.orm.manager import engine, rebuild_database, session
-from aop2db.orm.models import Aop, AopKer, AopKeyEvent, AopStressor, BiologicalAction, BiologicalEvent, \
-    BiologicalObject, BiologicalProcess, CellTerm, Chemical, KeyEvent, KeyEventRelationship, LifeStage, LifeStageAop, \
-    LifeStageKeyEvent, LifeStageKeyEventRelationship, OrganTerm, Sex, SexAop, SexKeyEvent, SexKeyEventRelationship, \
-    Stressor, Synonym, Taxonomy, TaxonomyAop, TaxonomyKeyEvent, TaxonomyKeyEventRelationship
-
 from sqlalchemy import insert
 from tqdm import tqdm
 
+from aop2db.constants import (
+    AOP_HASH,
+    AOP_ID,
+    AOP_JSON,
+    AOP_JSONS,
+    AOP_XML_DOWNLOAD,
+    APPLICABILITY,
+    CREATION,
+    CREATION_TIMESTAMP,
+    EVIDENCE,
+    ID,
+    KE_JSON,
+    LAST_MODIFIED,
+    LIFESTAGE,
+    LM_TIMESTAMP,
+    REFERENCES,
+    SEX,
+    STRESSOR_JSON,
+    TAX_ID_LOOKUP,
+    TAXONOMY,
+)
+from aop2db.defaults import AOP_DIR, AOP_XML_FILE, TAXONOMY_CACHE
+from aop2db.orm.manager import engine, rebuild_database, session
+from aop2db.orm.models import (
+    Aop,
+    AopKer,
+    AopKeyEvent,
+    AopStressor,
+    BiologicalAction,
+    BiologicalEvent,
+    BiologicalObject,
+    BiologicalProcess,
+    CellTerm,
+    Chemical,
+    KeyEvent,
+    KeyEventRelationship,
+    LifeStage,
+    LifeStageAop,
+    LifeStageKeyEvent,
+    LifeStageKeyEventRelationship,
+    OrganTerm,
+    Sex,
+    SexAop,
+    SexKeyEvent,
+    SexKeyEventRelationship,
+    Stressor,
+    Synonym,
+    Taxonomy,
+    TaxonomyAop,
+    TaxonomyKeyEvent,
+    TaxonomyKeyEventRelationship,
+)
+
 logger = logging.getLogger(__name__)
 
-__all__ = ['import_aop_data']
+__all__ = ["import_aop_data"]
 
 
 def import_aop_data() -> None:
@@ -61,7 +103,9 @@ class AopImporter:
     def __init__(self):
         """Init method."""
         # Get all AOP data as dict
-        self.aop_data: dict = self.__xml_to_dict()['data']
+
+        self.aop_data: dict = self.__xml_to_dict()["data"]
+        self.json_data = self.create_json_data_mapper()
         self.session = session()  # SQL session
 
         self.organ_terms: dict = dict()  # Filled during import_key_events
@@ -75,6 +119,29 @@ class AopImporter:
         self.current_sex_cache = set()
         self.current_ls_cache = set()
 
+    def create_json_data_mapper(self) -> dict:
+        """Create dictionary with JSON data mapper."""
+        self.__download_aop_json()  # Download JSON files
+        json_data_mapper = dict()
+        for file_name in (AOP_JSON, KE_JSON, STRESSOR_JSON):
+            file_path = AOP_DIR.joinpath(Path(file_name).name)
+
+            with file_path.open() as jsonf:
+                content = json.load(jsonf)
+
+            if file_name == STRESSOR_JSON:
+                mapper = dict()
+                for entry in content:
+                    for chemical in entry["chemicals"]:
+                        mapper[chemical["casrn"].strip()] = entry["id"]
+
+            else:
+                mapper = {entry["title"].strip(): entry["id"] for entry in content}
+
+            json_data_mapper[file_name] = mapper
+
+        return json_data_mapper
+
     def import_aop_data(self):
         """Import downloaded AOP data into database."""
         self.import_bio_classes()  # Dict => {bio class: inserted count}
@@ -106,10 +173,13 @@ class AopImporter:
         aop_entries = []
         for aop in tqdm(aop_data, total=len(aop_data), desc="Importing AOPs"):
             # Discard unused columns
-            aop.pop("overall-assessment", None)  # TODO decide later if we want to include them
+            aop.pop(
+                "overall-assessment", None
+            )  # TODO decide later if we want to include them
 
             # Extract useful information
-            aop[AOP_ID] = aop.pop(ID)
+            aop[AOP_ID] = self.__get_aop_id(AOP_JSON, aop["title"])
+            aop[AOP_HASH] = aop.pop(ID)
             aop[LAST_MODIFIED] = datetime.fromisoformat(aop.pop(LM_TIMESTAMP, None))
             aop[CREATION] = datetime.fromisoformat(aop.pop(CREATION_TIMESTAMP, None))
             aop[REFERENCES] = aop.pop(REFERENCES)
@@ -121,7 +191,9 @@ class AopImporter:
             kes = {
                 "key_events": aop.pop("key-events", None),
                 "aos": aop.pop("adverse-outcome", None),  # Adverse Outcomes
-                "mies": aop.pop("molecular-initiating-event", None),  # Molecular Initiating Events
+                "mies": aop.pop(
+                    "molecular-initiating-event", None
+                ),  # Molecular Initiating Events
             }
 
             # Create AOP object
@@ -133,9 +205,13 @@ class AopImporter:
                 aop_entry = self.__parse_applicability(applicability, aop_entry)
 
             if stressors:
-                aop_entry = self.__extract_aop_stressors(stressors, aop_entry, stressor_mapper)
+                aop_entry = self.__extract_aop_stressors(
+                    stressors, aop_entry, stressor_mapper
+                )
 
-            aop_entry = self.__extract_aop_key_events(kes, aop_entry, ke_mapper)  # Method checks if data present
+            aop_entry = self.__extract_aop_key_events(
+                kes, aop_entry, ke_mapper
+            )  # Method checks if data present
 
             if kers:
                 aop_entry = self.__extract_aop_kers(kers, aop_entry, ker_mapper)
@@ -144,24 +220,33 @@ class AopImporter:
 
         return aop_entries
 
-    def __extract_aop_key_events(self,
-                                 ke_dict: Dict[str, Union[dict, list]],
-                                 aop_entry: Aop,
-                                 key_event_mapper: dict) -> Aop:
+    def __extract_aop_key_events(
+        self,
+        ke_dict: Dict[str, Union[dict, list]],
+        aop_entry: Aop,
+        key_event_mapper: dict,
+    ) -> Aop:
         """Extract and label all key event associations.
 
         This includes Adverse Outcomes, Molecular Initiating Events, and other Key Events.
         """
         ke_tags = {
-            "mies": {EVIDENCE: "evidence-supporting-chemical-initiation", "ke_type": "molecular_initaiting_event"},
+            "mies": {
+                EVIDENCE: "evidence-supporting-chemical-initiation",
+                "ke_type": "molecular_initaiting_event",
+            },
             "aos": {EVIDENCE: "examples", "ke_type": "adverse_outcome"},
         }
 
         for ke_type, ke_data in ke_dict.items():
             if ke_data:
                 if ke_type == "key_events":
-                    ke_aop_ids = [ke[ID] for ke in self._listify(ke_data['key-event'])]
-                    ke_objs = [key_event_mapper[ke_aop_id] for ke_aop_id in ke_aop_ids]
+                    ke_aop_hashes = [
+                        ke[ID] for ke in self._listify(ke_data["key-event"])
+                    ]
+                    ke_objs = [
+                        key_event_mapper[ke_aop_hash] for ke_aop_hash in ke_aop_hashes
+                    ]
                     for ke_obj in ke_objs:
                         aop_ke_asso = AopKeyEvent(key_event_type="key_event")
                         aop_ke_asso.key_event = ke_obj
@@ -170,40 +255,52 @@ class AopImporter:
                 else:  # Either mies (molecular initiating events) or aos (adverse outcomes)
                     ke_data = self._listify(ke_data)
                     for special_event in ke_data:
-                        ke_aop_id = special_event["@key-event-id"]
+                        ke_aop_hash = special_event["@key-event-id"]
 
                         key_event_type = ke_tags[ke_type]["ke_type"]
                         evidence_tag = ke_tags[ke_type][EVIDENCE]
 
                         evidence = special_event[evidence_tag]
 
-                        ke_obj = key_event_mapper[ke_aop_id]
-                        aop_ke_asso = AopKeyEvent(key_event_type=key_event_type, evidence=evidence)
+                        ke_obj = key_event_mapper[ke_aop_hash]
+                        aop_ke_asso = AopKeyEvent(
+                            key_event_type=key_event_type, evidence=evidence
+                        )
                         aop_ke_asso.key_event = ke_obj
                         aop_entry.key_events.append(aop_ke_asso)
 
         return aop_entry
 
-    def __extract_aop_stressors(self, stressor_dict: dict, aop_entry: Aop, stressor_mapper: dict) -> Aop:
+    def __extract_aop_stressors(
+        self, stressor_dict: dict, aop_entry: Aop, stressor_mapper: dict
+    ) -> Aop:
         """Extract and add stressor_mapper to AOP entry."""
         stressors = self._listify(stressor_dict["aop-stressor"])
         for stressor in stressors:
-            stressor_aop_id = stressor["@stressor-id"]
-            stressor_asso = AopStressor(description=stressor["description"], evidence=stressor[EVIDENCE])
-            stressor_asso.stressor = stressor_mapper[stressor_aop_id]
+            stressor_aop_hash = stressor["@stressor-id"]
+            stressor_asso = AopStressor(
+                description=stressor["description"], evidence=stressor[EVIDENCE]
+            )
+            stressor_asso.stressor = stressor_mapper[stressor_aop_hash]
             aop_entry.stressors.append(stressor_asso)
 
         return aop_entry
 
-    def __extract_aop_kers(self, ker_dict: dict, aop_entry: Aop, ker_mapper: dict) -> Aop:
+    def __extract_aop_kers(
+        self, ker_dict: dict, aop_entry: Aop, ker_mapper: dict
+    ) -> Aop:
         """Extract and add key event relationships to AOP entry."""
         ker_list = self._listify(ker_dict["relationship"])
         for ker in ker_list:
-            ker_aop_id = ker[ID]
-            ker_asso = AopKer(adjacency=ker["adjacency"],
-                              quantitative_understanding_value=ker["quantitative-understanding-value"],
-                              evidence=ker[EVIDENCE])
-            ker_asso.ker = ker_mapper[ker_aop_id]
+            ker_aop_hash = ker[ID]
+            ker_asso = AopKer(
+                adjacency=ker["adjacency"],
+                quantitative_understanding_value=ker[
+                    "quantitative-understanding-value"
+                ],
+                evidence=ker[EVIDENCE],
+            )
+            ker_asso.ker = ker_mapper[ker_aop_hash]
             aop_entry.kers.append(ker_asso)
 
         return aop_entry
@@ -225,23 +322,29 @@ class AopImporter:
         Key events need to have been imported already.
         """
         logger.info("Parsing key event relationships")
-        kers = self.aop_data['key-event-relationship']
+        kers = self.aop_data["key-event-relationship"]
         key_events = self.__get_aop_mapper(KeyEvent)
 
         ker_entires = []
 
-        for ker in tqdm(kers, total=len(kers), desc="Importing key events relationships"):
-            ker[AOP_ID] = ker.pop(ID)
+        for ker in tqdm(
+            kers, total=len(kers), desc="Importing key events relationships"
+        ):
+            ker[AOP_HASH] = ker.pop(ID)
             ker[REFERENCES] = ker.pop(REFERENCES)
             ker = self.__extract_weight_of_evidence_values(ker)
             ker[LAST_MODIFIED] = datetime.fromisoformat(ker.pop(LM_TIMESTAMP, None))
             ker[CREATION] = datetime.fromisoformat(ker.pop(CREATION_TIMESTAMP, None))
-            ker["quantitative_understanding"] = ker.pop("quantitative-understanding")["description"]  # Get text
+            ker["quantitative_understanding"] = ker.pop("quantitative-understanding")[
+                "description"
+            ]  # Get text
             applicability = ker.pop("taxonomic-applicability", None)
             related_kes = ker.pop("title")
 
             # Create KER obj and add up/down stream KEs
-            ker = self.__standardize_keys(ker)  # only need to fix "evidence-supporting-taxonomic-applicability"
+            ker = self.__standardize_keys(
+                ker
+            )  # only need to fix "evidence-supporting-taxonomic-applicability"
             ker_entry = KeyEventRelationship(**ker)
             ker_entry.up_event = key_events[related_kes["upstream-id"]]
             ker_entry.down_event = key_events[related_kes["downstream-id"]]
@@ -257,8 +360,12 @@ class AopImporter:
         """Update the entries in the Sex table."""
         new_imports = []
         for sex_key, sex_entry in self.sex_entries.items():
-            if sex_key not in self.current_sex_cache:  # Sex key is new and needs to be imported
-                self.current_sex_cache.add(sex_key)  # Add to current cache to indicate it is imported
+            if (
+                sex_key not in self.current_sex_cache
+            ):  # Sex key is new and needs to be imported
+                self.current_sex_cache.add(
+                    sex_key
+                )  # Add to current cache to indicate it is imported
                 new_imports.append(sex_entry)
 
         self.session.add_all(new_imports)
@@ -268,8 +375,12 @@ class AopImporter:
         """Update the entries in the LifeStage table."""
         new_imports = []
         for ls, ls_obj in self.life_stages.items():
-            if ls not in self.current_ls_cache:  # Life stage is new and needs to be imported
-                self.current_ls_cache.add(ls)  # Add to current cache to indicate it is imported
+            if (
+                ls not in self.current_ls_cache
+            ):  # Life stage is new and needs to be imported
+                self.current_ls_cache.add(
+                    ls
+                )  # Add to current cache to indicate it is imported
                 new_imports.append(ls_obj)
 
         self.session.add_all(new_imports)
@@ -281,8 +392,12 @@ class AopImporter:
         woe = ker_entry.pop("weight-of-evidence")
         ker_entry["evidence_value"] = woe["value"]
         ker_entry["evidence_biological_plausibility"] = woe["biological-plausibility"]
-        ker_entry["evidence_emperical_support_linkage"] = woe["emperical-support-linkage"]
-        ker_entry["evidence_uncertainties_or_inconsistencies"] = woe["uncertainties-or-inconsistencies"]
+        ker_entry["evidence_emperical_support_linkage"] = woe[
+            "emperical-support-linkage"
+        ]
+        ker_entry["evidence_uncertainties_or_inconsistencies"] = woe[
+            "uncertainties-or-inconsistencies"
+        ]
         return ker_entry
 
     def import_key_events(self) -> int:
@@ -310,13 +425,14 @@ class AopImporter:
         There are certain entries with a list of these combinations instead of a single combination!
         """
         logger.info("Parsing key events")
-        key_events = self.aop_data['key-event']
+        key_events = self.aop_data["key-event"]
         stressor_mapper = self.__get_aop_mapper(Stressor)
 
         ke_entires = []
 
         for ke in tqdm(key_events, total=len(key_events), desc="Importing key events"):
-            ke[AOP_ID] = ke.pop(ID)
+            ke[AOP_ID] = self.__get_aop_id(KE_JSON, ke["title"])
+            ke[AOP_HASH] = ke.pop(ID)
             ke[REFERENCES] = ke.pop(REFERENCES)
 
             # Pop out terms with table relationships
@@ -324,35 +440,43 @@ class AopImporter:
             organ_term_data = ke.pop("organ-term", None)
             bio_events_dict = ke.pop("biological-events", None)
             applicability = ke.pop(APPLICABILITY, None)
-            stressors = ke.pop('key-event-stressors', None)
+            stressors = ke.pop("key-event-stressors", None)
 
             standardized_ke_dict = self.__standardize_keys(ke)
             key_event = KeyEvent(**standardized_ke_dict)
 
             if cell_term_data:
-                self.__create_modify_organ_cell_object(cell_term_data, CellTerm, key_event)
+                self.__create_modify_organ_cell_object(
+                    cell_term_data, CellTerm, key_event
+                )
 
             if organ_term_data:
-                self.__create_modify_organ_cell_object(organ_term_data, OrganTerm, key_event)
+                self.__create_modify_organ_cell_object(
+                    organ_term_data, OrganTerm, key_event
+                )
 
             if bio_events_dict:
-                self.__parse_bio_events(bio_events_dict, ke[AOP_ID])
+                self.__parse_bio_events(bio_events_dict, ke[AOP_HASH])
 
             if applicability:
                 key_event = self.__parse_applicability(applicability, key_event)
 
             if stressors:
-                key_event = self.__add_stressor_to_ke(stressors, key_event, stressor_mapper)
+                key_event = self.__add_stressor_to_ke(
+                    stressors, key_event, stressor_mapper
+                )
 
             ke_entires.append(key_event)
 
         return ke_entires
 
-    def __add_stressor_to_ke(self, stressors: dict, key_event: KeyEvent, mapper: dict) -> KeyEvent:
+    def __add_stressor_to_ke(
+        self, stressors: dict, key_event: KeyEvent, mapper: dict
+    ) -> KeyEvent:
         """Get stressor obj and append to key event. Returns modified key event obj."""
-        stressor_data = self._listify(stressors['key-event-stressor'])
+        stressor_data = self._listify(stressors["key-event-stressor"])
         stressor_ids = [stressor["@stressor-id"] for stressor in stressor_data]
-        stressor_objs = [mapper[aop_id] for aop_id in stressor_ids]
+        stressor_objs = [mapper[aop_hash] for aop_hash in stressor_ids]
         key_event.stressors = stressor_objs
         return key_event
 
@@ -362,11 +486,17 @@ class AopImporter:
             evidence_lines = self._listify(applicability_data[TAXONOMY])
 
             for evidence in evidence_lines:
-                tax_aop_id = evidence["@taxonomy-id"]
+                tax_aop_hash = evidence["@taxonomy-id"]
                 tax_evidence = evidence[EVIDENCE]
-                tax_asso = asso_class(evidence=tax_evidence)  # Create new association object
-                tax_asso.taxonomy = self.taxonomies[tax_aop_id]  # Set Tax entry to mapped Taxonomy obj
-                new_entry.taxonomies.append(tax_asso)  # Append association object to KeyEvent obj
+                tax_asso = asso_class(
+                    evidence=tax_evidence
+                )  # Create new association object
+                tax_asso.taxonomy = self.taxonomies[
+                    tax_aop_hash
+                ]  # Set Tax entry to mapped Taxonomy obj
+                new_entry.taxonomies.append(
+                    tax_asso
+                )  # Append association object to KeyEvent obj
 
         if SEX in applicability_data:
             asso_class = self.class_mapper[SEX][type(new_entry)]
@@ -413,15 +543,23 @@ class AopImporter:
 
         This depends on first importing key events to generate the bio_events attribute.
         """
-        bio_mapper = self.__query_bio_classes()  # Bio class AOP IDs as keys and Bio class Objs as values
+        bio_mapper = (
+            self.__query_bio_classes()
+        )  # Bio class AOP IDs as keys and Bio class Objs as values
         key_event_mapper = self.__get_aop_mapper(KeyEvent)
 
         bio_event_entries = []
-        for be_identifier, ke_ids in tqdm(self.bio_events.items(),
-                                          total=len(self.bio_events),
-                                          desc="Importing bio events"):
-            bio_class_objs = [bio_mapper[bio_key] for bio_key in be_identifier]  # List of bio class objects
-            key_events = [key_event_mapper[ke_id] for ke_id in ke_ids]  # List of KeyEvent objects
+        for be_identifier, ke_ids in tqdm(
+            self.bio_events.items(),
+            total=len(self.bio_events),
+            desc="Importing bio events",
+        ):
+            bio_class_objs = [
+                bio_mapper[bio_key] for bio_key in be_identifier
+            ]  # List of bio class objects
+            key_events = [
+                key_event_mapper[ke_id] for ke_id in ke_ids
+            ]  # List of KeyEvent objects
 
             new_be = BiologicalEvent()
             for bio_class_obj in bio_class_objs:  # Compile BiologicalEvent
@@ -443,20 +581,26 @@ class AopImporter:
         self.session.add_all(bio_event_entries)
         self.session.commit()
 
-    def __parse_bio_events(self, bio_events: dict, key_event_aop_id: str):
+    def __parse_bio_events(self, bio_events: dict, key_event_aop_hash: str):
         """Parse bio event information in key events."""
         logger.info("Parsing biological events")
-        bes = self._listify(bio_events['biological-event'])
+        bes = self._listify(bio_events["biological-event"])
         for be in bes:
-            be_key = self.__create_unique_key(be)  # Sort bio AOP IDs into tuple and use as key
+            be_key = self.__create_unique_key(
+                be
+            )  # Sort bio AOP IDs into tuple and use as key
 
             if be_key in self.bio_events:
-                self.bio_events[be_key].append(key_event_aop_id)  # Add KeyEvent obj to list
+                self.bio_events[be_key].append(
+                    key_event_aop_hash
+                )  # Add KeyEvent obj to list
 
             else:
-                self.bio_events[be_key] = [key_event_aop_id]  # Start new list
+                self.bio_events[be_key] = [key_event_aop_hash]  # Start new list
 
-    def __create_modify_organ_cell_object(self, term_values: dict, table, key_event: KeyEvent):
+    def __create_modify_organ_cell_object(
+        self, term_values: dict, table, key_event: KeyEvent
+    ):
         """Sanitize keys and values for an Organ or Cell Term and create/modify ORM obj.
 
         This method checks whether there already exists an ORM object to be added to the DB in the cache. If not,
@@ -465,12 +609,20 @@ class AopImporter:
         """
         term_cache_mapper = {CellTerm: self.cell_terms, OrganTerm: self.organ_terms}
         term_values["source_id"] = self.__split_source_id(term_values.pop("source-id"))
-        unique_key = (term_values["name"], term_values["source"], term_values["source_id"])
+        unique_key = (
+            term_values["name"],
+            term_values["source"],
+            term_values["source_id"],
+        )
 
-        if unique_key in term_cache_mapper[table]:  # Source/Source ID/Name already in term cache
+        if (
+            unique_key in term_cache_mapper[table]
+        ):  # Source/Source ID/Name already in term cache
             term_obj = term_cache_mapper[table][unique_key]
             term_obj.key_events.append(key_event)  # Add key event to obj
-            term_cache_mapper[table][unique_key] = term_obj  # Overwrite old cache obj with new modified one
+            term_cache_mapper[table][
+                unique_key
+            ] = term_obj  # Overwrite old cache obj with new modified one
 
         else:
             new_orm_obj = table(**term_values)  # Create new ORM obj
@@ -505,29 +657,38 @@ class AopImporter:
 
         chem_mapper = self.__get_aop_mapper(Chemical)
         if not chem_mapper:  # Chemicals weren't imported
-            logger.warning("""Attempting to import stressor data into DB before chemicals. Stressors will not be
-            properly mapped to chemicals!""")
+            logger.warning(
+                """Attempting to import stressor data into DB before chemicals. Stressors will not be
+            properly mapped to chemicals!"""
+            )
 
         stressor_entries = []
-        stressors = self.aop_data['stressor']
+        stressors = self.aop_data["stressor"]
         for stressor in stressors:
-            stressor[AOP_ID] = stressor.pop(ID)
+            # stressor[AOP_ID] = self.__get_aop_id(STRESSOR_JSON, stressor["casrn"])  #TODO Parse chemicals for CASRN
+            stressor[AOP_HASH] = stressor.pop(ID)
             chemical_data = stressor.pop("chemicals", None)
-            stressor[CREATION] = datetime.fromisoformat(stressor.pop(CREATION_TIMESTAMP))
+            stressor[CREATION] = datetime.fromisoformat(
+                stressor.pop(CREATION_TIMESTAMP)
+            )
             stressor[LAST_MODIFIED] = datetime.fromisoformat(stressor.pop(LM_TIMESTAMP))
             standardized_stressor_dict = self.__standardize_keys(stressor)
             stressor_entry = Stressor(**standardized_stressor_dict)
 
             if chemical_data:
-                chem_inits = self._listify(chemical_data['chemical-initiator'])  # Map to list
+                chem_inits = self._listify(
+                    chemical_data["chemical-initiator"]
+                )  # Map to list
 
                 chemical_objs = []
                 for val in chem_inits:
                     try:
-                        chemical_objs.append(chem_mapper[val['@chemical-id']])
+                        chemical_objs.append(chem_mapper[val["@chemical-id"]])
 
                     except KeyError:
-                        logger.error(f"{val['@chemical-id']} not found in chemical dataset. Are chemicals imported?")
+                        logger.error(
+                            f"{val['@chemical-id']} not found in chemical dataset. Are chemicals imported?"
+                        )
 
                 stressor_entry.chemicals = chemical_objs
 
@@ -535,6 +696,11 @@ class AopImporter:
 
         return stressor_entries
 
+    def __get_aop_id(self, json_url: str, map_key: str) -> Optional[int]:
+        """Get the AOP ID from the downloaded JSON content."""
+        if map_key in self.json_data[json_url]:
+            return self.json_data[json_url][map_key]
+
     def import_chemicals(self) -> dict:
         """Import parsed chemical data into database and return number of rows imported for chemicals and synonyms."""
         parsed_chemicals = self.__parse_chemicals()
@@ -542,12 +708,14 @@ class AopImporter:
         synonym_rows = []
         with engine.connect() as conn:
             # Add chem to DB one at a time to get chem IDs for synonyms
-            for aop_id, chem_data in tqdm(parsed_chemicals.items(),
-                                          total=len(parsed_chemicals),
-                                          desc="Importing chemicals and synonyms"):
+            for aop_hash, chem_data in tqdm(
+                parsed_chemicals.items(),
+                total=len(parsed_chemicals),
+                desc="Importing chemicals and synonyms",
+            ):
                 logger.info("Importing chemicals")
                 # Format data
-                chem_data[AOP_ID] = aop_id
+                chem_data[AOP_HASH] = aop_hash
                 synonyms = chem_data.pop("synonyms")
                 standardize_data = self.__standardize_keys(chem_data)
                 standardize_data["name"] = standardize_data.pop("preferred_name")
@@ -561,21 +729,24 @@ class AopImporter:
             logger.info("Importing synonyms")
             synonyms_insert_result = conn.execute(insert(Synonym), synonym_rows)
 
-        return {"chemicals": chem_result.rowcount, "synonyms": synonyms_insert_result.rowcount}
+        return {
+            "chemicals": chem_result.rowcount,
+            "synonyms": synonyms_insert_result.rowcount,
+        }
 
     def __parse_chemicals(self) -> dict:
         """Extract chemicals into dictionary."""
         logger.info("Parsing chemicals")
-        chemicals = self.aop_data['chemical']
+        chemicals = self.aop_data["chemical"]
         parsed_chemicals = dict()
         for chem in chemicals:
-            chem_id = chem.pop('@id')
+            chem_id = chem.pop("@id")
 
             synonyms = []
             if "synonyms" in chem:
-                synonyms = chem['synonyms']['synonym']
+                synonyms = chem["synonyms"]["synonym"]
 
-            chem['synonyms'] = [synonyms] if isinstance(synonyms, str) else synonyms
+            chem["synonyms"] = [synonyms] if isinstance(synonyms, str) else synonyms
 
             parsed_chemicals[chem_id] = chem
 
@@ -587,17 +758,28 @@ class AopImporter:
         This includes bio object, action, and process.
         """
         bio_class_metadata = {
-            "biological_object": {"aop_key": "biological-object", "table": BiologicalObject},
-            "biological_action": {"aop_key": "biological-action", "table": BiologicalAction},
-            "biological_process": {"aop_key": "biological-process", "table": BiologicalProcess},
+            "biological_object": {
+                "aop_key": "biological-object",
+                "table": BiologicalObject,
+            },
+            "biological_action": {
+                "aop_key": "biological-action",
+                "table": BiologicalAction,
+            },
+            "biological_process": {
+                "aop_key": "biological-process",
+                "table": BiologicalProcess,
+            },
         }
 
         parsed_bio_data = self.__parse_bio_classes(bio_class_metadata)
 
         counts = dict()
-        for bio_class, parsed_entries in tqdm(parsed_bio_data.items(),
-                                              total=len(parsed_bio_data),
-                                              desc="Importing bio tables"):
+        for bio_class, parsed_entries in tqdm(
+            parsed_bio_data.items(),
+            total=len(parsed_bio_data),
+            desc="Importing bio tables",
+        ):
             table = bio_class_metadata[bio_class]["table"]
             inserted = self.__import_simple_rows(entries=parsed_entries, table=table)
             logger.info(f"Inserted {inserted} rows into {bio_class}")
@@ -608,16 +790,19 @@ class AopImporter:
     def __parse_bio_classes(self, bio_classes: dict) -> dict:
         """Extract bio class data into dictionaries. Returns dict of metadata information."""
         parsed_bio_data = dict()
-        for bio_class, metadata in bio_classes.items():  # All bio classes structured similarly
+        for (
+            bio_class,
+            metadata,
+        ) in bio_classes.items():  # All bio classes structured similarly
             logger.info(f"Parsing {bio_class}")
             parsed_entries = dict()
             aop_entries = self.aop_data[metadata["aop_key"]]
 
             for entry in aop_entries:  # Iterate through class entries
-                aop_id = entry.pop(ID)
+                aop_hash = entry.pop(ID)
                 source_id = entry.pop("source-id")
                 entry["source_id"] = self.__split_source_id(source_id)
-                parsed_entries[aop_id] = entry
+                parsed_entries[aop_hash] = entry
 
             parsed_bio_data[bio_class] = parsed_entries
 
@@ -625,26 +810,30 @@ class AopImporter:
 
     def __parse_taxonomies(self) -> dict:
         """Extract taxonomies into dictionary."""
-        taxos = self.aop_data['taxonomy']
+        taxos = self.aop_data["taxonomy"]
         species_names = self.__get_tax_id_cache()
         logger.info("Parsing taxonomies")
 
         parsed_taxos = dict()
         for taxo in taxos:
             entry_id = taxo.pop(ID)
-            source_id = taxo.pop('source-id')
+            source_id = taxo.pop("source-id")
             parsed_id = self.__split_source_id(source_id)
 
             if parsed_id.startswith("WikiUser"):
-                taxo['source_id'] = parsed_id  # Only use "source_id" with WikiUser entries, all others are tax IDs
+                taxo[
+                    "source_id"
+                ] = parsed_id  # Only use "source_id" with WikiUser entries, all others are tax IDs
                 taxo["tax_id"] = None
                 taxo["species"] = None
 
             else:
-                taxo['source_id'] = None
+                taxo["source_id"] = None
                 taxo["tax_id"] = int(parsed_id)
 
-                if parsed_id not in species_names:  # Check cache first and if not there, query EnsEMBL
+                if (
+                    parsed_id not in species_names
+                ):  # Check cache first and if not there, query EnsEMBL
                     species_name = self.__lookup_species(parsed_id)
                     taxo["species"] = species_name
                     species_names[parsed_id] = species_name
@@ -655,7 +844,7 @@ class AopImporter:
             parsed_taxos[entry_id] = taxo
 
         # Write parsed/queried taxonomies to cache file
-        with open(TAXONOMY_CACHE, 'w') as tax_file:
+        with open(TAXONOMY_CACHE, "w") as tax_file:
             json.dump(species_names, tax_file)
 
         return parsed_taxos
@@ -677,8 +866,8 @@ class AopImporter:
     def __import_simple_rows(entries: dict, table) -> int:
         """Import entries into specified table. Only works with simple tables. Returns number of rows inserted."""
         table_rows = []
-        for aop_id, entry_data in entries.items():
-            entry_data[AOP_ID] = aop_id
+        for aop_hash, entry_data in entries.items():
+            entry_data[AOP_HASH] = aop_hash
             table_rows.append(entry_data)
 
         with engine.connect() as conn:
@@ -690,8 +879,8 @@ class AopImporter:
     def __get_tax_id_cache() -> dict:
         """Import cached taxonomy IDs if JSON exists, else returns empty dict."""
         tax_cache = dict()
-        if os.path.isfile(TAXONOMY_CACHE):
-            with open(TAXONOMY_CACHE, 'r') as tax_file:
+        if Path(TAXONOMY_CACHE).is_file():
+            with open(TAXONOMY_CACHE, "r") as tax_file:
                 tax_cache = json.load(tax_file)
 
         return tax_cache
@@ -702,7 +891,7 @@ class AopImporter:
         logger.info(f"Querying EnsEMBL for {tax_id}")
         resp = requests.get(TAX_ID_LOOKUP.format(tax_id))
         if resp.ok:
-            species = resp.json()['name']
+            species = resp.json()["name"]
             return species
 
     @staticmethod
@@ -717,7 +906,7 @@ class AopImporter:
 
     def __get_aop_mapper(self, table) -> dict:
         """Generate a mapper with AOP IDs as keys and ORM objs as values for a specified table."""
-        results = self.session.query(table.aop_id, table).all()
+        results = self.session.query(table.aop_hash, table).all()
         return {x[0]: x[1] for x in results}
 
     @staticmethod
@@ -725,7 +914,9 @@ class AopImporter:
         """Split the source ID to remove the prefix if it exists."""
         split_terms = re.split(r"_|:", source_id)
 
-        if len(split_terms) > 1 and split_terms[0] != "WikiUser":  # WikiUser for taxonomy
+        if (
+            len(split_terms) > 1 and split_terms[0] != "WikiUser"
+        ):  # WikiUser for taxonomy
             return split_terms[1]
 
         else:  # No delim in term
@@ -734,17 +925,37 @@ class AopImporter:
     @staticmethod
     def __download_aop_xml():
         """Download the default AOP XML file to data directory."""
-        if not os.path.isfile(AOP_XML_FILE):
+        if not Path(AOP_XML_FILE).is_file():
             logger.info(f"Downloading {AOP_XML_DOWNLOAD}")
             resp = requests.get(AOP_XML_DOWNLOAD)
             if resp.ok:
-                with open(AOP_XML_FILE, 'wb') as aopf:
+                with open(AOP_XML_FILE, "wb") as aopf:
                     aopf.write(resp.content)
-                logger.info(f"{os.path.basename(AOP_XML_DOWNLOAD)} saved to {AOP_XML_FILE}")
+                logger.info(f"{Path(AOP_XML_DOWNLOAD).name} saved to {AOP_XML_FILE}")
 
             else:
                 logger.warning(f"Could not download {AOP_XML_DOWNLOAD}!")
 
+    @staticmethod
+    def __download_aop_json():
+        """Download the AOP JSON to get AOP IDs."""
+        downloaded_files = [file_path.name for file_path in AOP_DIR.iterdir()]
+        for download_path in AOP_JSONS:
+            json_file_name = Path(download_path).name
+            if json_file_name not in downloaded_files:
+                resp = requests.get(download_path)
+
+                if resp.ok:
+                    with open(AOP_DIR.joinpath(json_file_name), "w") as jsonf:
+                        json.dump(resp.json(), jsonf)
+
+                    logger.info(
+                        f"{json_file_name} saved to {AOP_DIR.joinpath(json_file_name)}"
+                    )
+
+                else:
+                    logger.warning(f"Could not download {json_file_name}!")
+
     @staticmethod
     def __create_unique_key(value_dict: dict) -> tuple:
         """Generate a qunique key by taking values form a dictionary, sorting them, and return the tuple."""
@@ -770,19 +981,23 @@ class AopImporter:
     def __xml_to_dict(self) -> dict:
         """Read and convert AOP XML to a dictionary, then write to JSON if not there. Returns XML content as dict."""
         self.__download_aop_xml()
-        json_file_path = AOP_XML_FILE + ".json"
-        if not os.path.isfile(json_file_path):
-            with gzip.open(AOP_XML_FILE, 'rb') as aopf:
+        json_file_path = str(AOP_XML_FILE) + ".json"
+        if not Path(json_file_path).is_file():
+            with gzip.open(AOP_XML_FILE, "rb") as aopf:
                 content = aopf.read()
 
             aop_dict = xmltodict.parse(content)
 
-            with open(json_file_path, 'w') as aopjson:
+            with open(json_file_path, "w") as aopjson:
                 json.dump(aop_dict, aopjson)
 
             logger.info(f"AOP JSON written to {json_file_path}")
 
-        with open(json_file_path, 'r') as aopjson:
+        with open(json_file_path, "r") as aopjson:
             content_as_json = json.load(aopjson)
 
         return content_as_json
+
+
+if __name__ == "__main__":
+    import_aop_data()
