@@ -1,9 +1,12 @@
 """String definitions."""
 
+from . import LATEST_XML_VERSION
+
 DATABASE = "DATABASE"
 
 ID = "@id"
 AOP_ID = "aop_id"
+AOP_HASH = "aop_hash"
 APPLICABILITY = "applicability"
 TAXONOMY = "taxonomy"
 EVIDENCE = "evidence"
@@ -18,5 +21,12 @@ REFERENCES = "references"
 # API
 TAX_ID_LOOKUP = "https://rest.ensembl.org/taxonomy/id/{}?content-type=application/json"
 
-# AOP Download
-AOP_XML_DOWNLOAD = "https://aopwiki.org/downloads/aop-wiki-xml-2021-07-01.gz"
+# AOP Downloads
+AOP_XML_DOWNLOAD = f"https://aopwiki.org/downloads/aop-wiki-xml-{LATEST_XML_VERSION}.gz"
+
+AOP_JSON = "https://aopwiki.org/aops.json"
+KE_JSON = "https://aopwiki.org/events.json"
+# KER_JSON = "https://aopwiki.org/relationships.json"  # No useful way to map information
+STRESSOR_JSON = "https://aopwiki.org/stressors.json"
+
+AOP_JSONS = (AOP_JSON, KE_JSON, STRESSOR_JSON)
